from faststream.redis import RedisBroker

from approck_messaging.models.message import TransportMessage


class Publisher:
    def __init__(self, broker: RedisBroker, stream: str) -> None:
        self.broker = broker
        self.publisher = self.broker.publisher(stream=stream)

    @classmethod
    def from_uri(cls, redis_uri: str, stream: str) -> "Publisher":
        return cls(broker=RedisBroker(redis_uri), stream=stream)

    async def send_message(self, message: TransportMessage) -> None:
        await self.publisher.publish(message)



from typing import Callable, Any

from faststream import context
from faststream.broker.types import P_HandlerParams, T_HandlerReturn
from faststream.broker.wrapper import HandlerCallWrapper
from faststream.redis import RedisBroker


class Subscriber:
    def __init__(self, broker: RedisBroker, **kwargs) -> None:
        self.broker = broker

        for key, value in kwargs.items():
            context.set_global(key, value)

    @classmethod
    def from_uri(cls, redis_uri: str, **kwargs) -> "Subscriber":
        return cls(broker=RedisBroker(redis_uri))

    def message(
        self, stream: str
    ) -> Callable[
        [Callable[P_HandlerParams, T_HandlerReturn]],
        HandlerCallWrapper[Any, P_HandlerParams, T_HandlerReturn],
    ]:
        return self.broker.subscriber(stream=stream)
